Description: <short summary of the patch>
 TODO: Put a short summary on the line above and replace this paragraph
 with a longer explanation of this change. Complete the meta-information
 with other relevant fields (see below for details). To make it easier, the
 information below has been extracted from the changelog. Adjust it or drop
 it.
 .
 acl2 (7.1-3) unstable; urgency=medium
 .
   * don't pre-allocate contiguous blocks in acl2.lisp
Author: Camm Maguire <camm@debian.org>

---
The information above should follow the Patch Tagging Guidelines, please
checkout http://dep.debian.net/deps/dep3/ to learn about the format. Here
are templates for supplementary fields that you might want to add:

Origin: <vendor|upstream|other>, <url of original patch>
Bug: <url in upstream bugtracker>
Bug-Debian: https://bugs.debian.org/<bugnumber>
Bug-Ubuntu: https://launchpad.net/bugs/<bugnumber>
Forwarded: <no|not-needed|url proving that it has been forwarded>
Reviewed-By: <name and email of someone who approved the patch>
Last-Update: <YYYY-MM-DD>

--- acl2-7.1.orig/acl2.lisp
+++ acl2-7.1/acl2.lisp
@@ -506,10 +506,10 @@
 
 ; Camm Maguire has suggested, on 9/22/2013, the following forms, which allowed
 ; him to complete an ACL2 regresssion using 2.6.10pre.
-#+gcl
-(progn
-  (si::allocate 'contiguous 15000 t)
-  (si::allocate-sgc 'contiguous 15000 100000 10))
+;; #+gcl
+;; (progn
+;;   (si::allocate 'contiguous 15000 t)
+;;   (si::allocate-sgc 'contiguous 15000 100000 10))
 
 ; The following avoids errors from extra right parentheses, but we leave it
 ; commented out since it doesn't seem important enough to merit messing around
